@@ -25,7 +25,6 @@ Test definitions for plainbox.impl.secure.providers.v1 module
 """
 
 from unittest import TestCase
-import os
 
 from plainbox.impl.job import JobDefinition
 from plainbox.impl.secure.config import Unset
@@ -470,7 +469,9 @@ class Provider1PlugInTests(TestCase):
         self.assertEqual(provider.whitelists_dir, None)
         self.assertEqual(provider.data_dir, None)
         self.assertEqual(provider.bin_dir, None)
+        self.assertEqual(provider.build_bin_dir, None)
         self.assertEqual(provider.locale_dir, None)
+        self.assertEqual(provider.base_dir, None)
 
     def test_provider_directories__w_location(self):
         """
@@ -483,7 +484,9 @@ class Provider1PlugInTests(TestCase):
         self.assertEqual(provider.whitelists_dir, "/some/directory/whitelists")
         self.assertEqual(provider.data_dir, "/some/directory/data")
         self.assertEqual(provider.bin_dir, "/some/directory/bin")
+        self.assertEqual(provider.build_bin_dir, "/some/directory/build/bin")
         self.assertEqual(provider.locale_dir, "/some/directory/locale")
+        self.assertEqual(provider.base_dir, "/some/directory")
 
     def test_provider_directories__w_location_w_no_dirs(self):
         """
@@ -497,7 +500,9 @@ class Provider1PlugInTests(TestCase):
         self.assertEqual(provider.whitelists_dir, None)
         self.assertEqual(provider.data_dir, None)
         self.assertEqual(provider.bin_dir, None)
+        self.assertEqual(provider.build_bin_dir, "/some/directory/build/bin")
         self.assertEqual(provider.locale_dir, None)
+        self.assertEqual(provider.base_dir, "/some/directory")
 
     def test_provider_directories__w_dirs(self):
         """
@@ -509,7 +514,9 @@ class Provider1PlugInTests(TestCase):
         self.assertEqual(provider.whitelists_dir, "/some/directory/whitelists")
         self.assertEqual(provider.data_dir, "/some/directory/data")
         self.assertEqual(provider.bin_dir, "/some/directory/bin")
+        self.assertEqual(provider.build_bin_dir, None)
         self.assertEqual(provider.locale_dir, "/some/directory/locale")
+        self.assertEqual(provider.base_dir, None)
 
 
 class WhiteListPlugInTests(TestCase):
@@ -639,12 +646,16 @@ class Provider1Tests(TestCase):
     DATA_DIR = "data-dir"
     BIN_DIR = "bin-dir"
     LOCALE_DIR = "locale-dir"
+    BASE_DIR = "base-dir"
 
     def setUp(self):
         self.provider = Provider1(
             self.NAME, self.VERSION, self.DESCRIPTION, self.SECURE,
             self.GETTEXT_DOMAIN, self.JOBS_DIR, self.WHITELISTS_DIR,
-            self.DATA_DIR, self.BIN_DIR, self.LOCALE_DIR)
+            self.DATA_DIR, self.BIN_DIR, self.LOCALE_DIR, self.BASE_DIR,
+            # We are using dummy job definitions so let's not shout about those
+            # being invalid in each test
+            validate=False)
 
     def test_repr(self):
         self.assertEqual(
@@ -717,19 +728,24 @@ class Provider1Tests(TestCase):
         """
         self.assertEqual(self.provider.locale_dir, self.LOCALE_DIR)
 
+    def test_base_dir(self):
+        """
+        Verify that Provider1.base_dir attribute is set correctly
+        """
+        self.assertEqual(self.provider.base_dir, self.BASE_DIR)
+
     def test_CHECKBOX_SHARE(self):
         """
         Verify that Provider1.CHECKBOX_SHARE is defined as the parent directory
         of data_dir
         """
-        self.assertEqual(self.provider.CHECKBOX_SHARE,
-                         os.path.join(self.DATA_DIR, ".."))
+        self.assertEqual(self.provider.CHECKBOX_SHARE, self.BASE_DIR)
 
-    def test_CHECKBOX_SHARE__without_data_dir(self):
+    def test_CHECKBOX_SHARE__without_base_dir(self):
         """
-        Verify that Provider1.CHECKBOX_SHARE is None without data_dir
+        Verify that Provider1.CHECKBOX_SHARE is None without base_dir
         """
-        self.provider._data_dir = None
+        self.provider._base_dir = None
         self.assertEqual(self.provider.CHECKBOX_SHARE, None)
 
     def test_extra_PYTHONPATH(self):
@@ -769,7 +785,7 @@ class Provider1Tests(TestCase):
         provider = Provider1(
             self.NAME, self.VERSION, self.DESCRIPTION, self.SECURE,
             self.GETTEXT_DOMAIN, self.JOBS_DIR, None,
-            self.DATA_DIR, self.BIN_DIR, self.LOCALE_DIR)
+            self.DATA_DIR, self.BIN_DIR, self.LOCALE_DIR, self.BASE_DIR)
         self.assertEqual(provider.get_builtin_whitelists(), [])
 
     def test_get_builtin_jobs__normal(self):
@@ -782,11 +798,11 @@ class Provider1Tests(TestCase):
             JobDefinitionPlugIn("/path/to/jobs1.txt", (
                 "id: a2\n"
                 "\n"
-                "id: a1\n"), self.provider),
+                "id: a1\n"), self.provider, validate=False),
             JobDefinitionPlugIn("/path/to/jobs2.txt", (
                 "id: a3\n"
                 "\n"
-                "id: a4\n"), self.provider)
+                "id: a4\n"), self.provider, validate=False)
         ]
         with self.provider._job_collection.fake_plugins(fake_plugins):
             job_list = self.provider.get_builtin_jobs()
@@ -817,7 +833,7 @@ class Provider1Tests(TestCase):
         provider = Provider1(
             self.NAME, self.VERSION, self.DESCRIPTION, self.SECURE,
             self.GETTEXT_DOMAIN, None, self.WHITELISTS_DIR,
-            self.DATA_DIR, self.BIN_DIR, self.LOCALE_DIR)
+            self.DATA_DIR, self.BIN_DIR, self.LOCALE_DIR, self.BASE_DIR)
         self.assertEqual(provider.get_builtin_jobs(), [])
 
     def test_load_all_jobs__normal(self):
@@ -830,11 +846,11 @@ class Provider1Tests(TestCase):
             JobDefinitionPlugIn("/path/to/jobs1.txt", (
                 "id: a2\n"
                 "\n"
-                "id: a1\n"), self.provider),
+                "id: a1\n"), self.provider, validate=False),
             JobDefinitionPlugIn("/path/to/jobs2.txt", (
                 "id: a3\n"
                 "\n"
-                "id: a4\n"), self.provider)
+                "id: a4\n"), self.provider, validate=False)
         ]
         with self.provider._job_collection.fake_plugins(fake_plugins):
             job_list, problem_list = self.provider.load_all_jobs()
@@ -852,7 +868,8 @@ class Provider1Tests(TestCase):
         """
         fake_plugins = [
             JobDefinitionPlugIn(
-                "/path/to/jobs1.txt", "id: working\n", self.provider)
+                "/path/to/jobs1.txt", "id: working\n", self.provider,
+                validate=False)
         ]
         fake_problems = [
             PlugInError("some problem"),
@@ -875,7 +892,7 @@ class Provider1Tests(TestCase):
         provider = Provider1(
             self.NAME, self.VERSION, self.DESCRIPTION, self.SECURE,
             self.GETTEXT_DOMAIN, self.JOBS_DIR, self.WHITELISTS_DIR,
-            self.DATA_DIR, None, self.LOCALE_DIR)
+            self.DATA_DIR, None, self.LOCALE_DIR, self.BASE_DIR)
         self.assertEqual(provider.get_all_executables(), [])
 
     @mock.patch("plainbox.impl.secure.providers.v1.gettext")
@@ -932,7 +949,7 @@ class Provider1Tests(TestCase):
         Provider1(
             self.NAME, self.VERSION, self.DESCRIPTION, self.SECURE,
             self.GETTEXT_DOMAIN, self.JOBS_DIR, self.WHITELISTS_DIR,
-            self.DATA_DIR, self.BIN_DIR, self.LOCALE_DIR)
+            self.DATA_DIR, self.BIN_DIR, self.LOCALE_DIR, self.BASE_DIR)
         mock_gettext.bindtextdomain.assert_called_once_with(
             self.GETTEXT_DOMAIN, self.LOCALE_DIR)
 
@@ -945,5 +962,6 @@ class Provider1Tests(TestCase):
         Provider1(
             self.NAME, self.VERSION, self.DESCRIPTION, self.SECURE,
             self.GETTEXT_DOMAIN, self.JOBS_DIR, self.WHITELISTS_DIR,
-            self.DATA_DIR, self.BIN_DIR, locale_dir=None)
+            self.DATA_DIR, self.BIN_DIR, locale_dir=None,
+            base_dir=self.BASE_DIR)
         self.assertEqual(mock_gettext.bindtextdomain.call_args_list, [])
