@@ -32,7 +32,10 @@ import tarfile
 import tempfile
 
 from plainbox.impl.secure.providers.v1 import Provider1Definition
+from plainbox.provider_manager import InstallCommand
+from plainbox.provider_manager import ManageCommand
 from plainbox.provider_manager import ProviderManagerTool
+from plainbox.provider_manager import manage_py_extension
 from plainbox.testing_utils.io import TestIO
 from plainbox.vendor import mock
 
@@ -56,13 +59,15 @@ class ProviderManagerToolTests(TestCase):
                 usage: manage.py [--help] [--version] [options]
 
                 positional arguments:
-                  {info,validate,develop,install,sdist,i18n}
+                  {info,validate,develop,install,sdist,i18n,build,clean}
                     info                display basic information about this provider
                     validate            perform various static analysis and validation
                     develop             install/remove this provider, only for development
                     install             install this provider in the system
                     sdist               create a source tarball
                     i18n                update, merge and build translation catalogs
+                    build               build provider specific executables from source
+                    clean               clean build results
 
                 optional arguments:
                   -h, --help            show this help message and exit
@@ -88,7 +93,7 @@ class ProviderManagerToolTests(TestCase):
             "[PlainBox Provider]\n"
             "description = description\n"
             "gettext_domain = domain\n"
-            "location = {prefix}/lib/plainbox-providers-1/2014.com.example.test\n"
+            "location = {prefix}/lib/plainbox-providers-1/2014.com.example:test\n"
             "name = 2014.com.example:test\n"
             "version = 1.0\n"
             "\n".format(prefix=prefix))
@@ -120,7 +125,7 @@ class ProviderManagerToolTests(TestCase):
             self.tmpdir + os.path.join("/foo", "lib", "plainbox-providers-1",
                                        "2014.com.example:test", "jobs",
                                        "jobs.txt"),
-            "name: dummy\nplugin: shell\ncommand: true\n")
+            "id: dummy\nplugin: shell\ncommand: true\n")
 
     def test_install__flat_partial(self):
         """
@@ -147,7 +152,6 @@ class ProviderManagerToolTests(TestCase):
             "description = description\n"
             "gettext_domain = domain\n"
             "jobs_dir = {prefix}/share/2014.com.example:test/jobs\n"
-            "locale_dir = {prefix}/share/locale\n"
             "name = 2014.com.example:test\n"
             "version = 1.0\n"
             "whitelists_dir = {prefix}/share/2014.com.example:test/whitelists\n"
@@ -156,7 +160,7 @@ class ProviderManagerToolTests(TestCase):
         self.assertFileContent(
             self.tmpdir + os.path.join(
                 prefix, "share", "2014.com.example:test", "jobs", "jobs.txt"),
-            "name: dummy\nplugin: shell\ncommand: true\n")
+            "id: dummy\nplugin: shell\ncommand: true\n")
         self.assertFileContent(
             self.tmpdir + os.path.join(
                 prefix, "share",  "2014.com.example:test", "whitelists",
@@ -189,6 +193,9 @@ class ProviderManagerToolTests(TestCase):
         self.assertTarballContent(
             tarball, "2014.com.example.test-1.0/bin/test.sh",
             "#!/bin/sh\n:\n")
+        self.assertTarballContent(
+            tarball, "2014.com.example.test-1.0/src/hello.c",
+            "int main() { return 0; }\n")
 
     def test_sdist(self):
         """
@@ -199,7 +206,7 @@ class ProviderManagerToolTests(TestCase):
             self.tmpdir, "dist", "2014.com.example.test-1.0.tar.gz")
         self.assertTarballContent(
             tarball, "2014.com.example.test-1.0/jobs/jobs.txt",
-            "name: dummy\nplugin: shell\ncommand: true\n")
+            "id: dummy\nplugin: shell\ncommand: true\n")
         self.assert_common_sdist(tarball)
 
     def test_sdist__partial(self):
@@ -288,7 +295,7 @@ class ProviderManagerToolTests(TestCase):
         """
         filename = os.path.join(self.tmpdir, "jobs", "broken.txt")
         with open(filename, "wt", encoding='UTF-8') as stream:
-            print("name: broken", file=stream)
+            print("id: broken", file=stream)
             print("plugin: shell", file=stream)
         with TestIO() as test_io:
             self.tool.main(["validate"])
@@ -304,7 +311,7 @@ class ProviderManagerToolTests(TestCase):
         """
         filename = os.path.join(self.tmpdir, "jobs", "broken.txt")
         with open(filename, "wt", encoding='UTF-8') as stream:
-            print("name: broken", file=stream)
+            print("id: broken", file=stream)
             print("plugin: magic", file=stream)
         with TestIO() as test_io:
             self.tool.main(["validate"])
@@ -322,7 +329,7 @@ class ProviderManagerToolTests(TestCase):
         """
         filename = os.path.join(self.tmpdir, "jobs", "broken.txt")
         with open(filename, "wt", encoding='UTF-8') as stream:
-            print("name: broken", file=stream)
+            print("id: broken", file=stream)
             print("plugin: manual", file=stream)
             print("description: broken job definition", file=stream)
             print("command: true", file=stream)
@@ -333,6 +340,23 @@ class ProviderManagerToolTests(TestCase):
                 "jobs/broken.txt:1-4: job '2014.com.example::broken', field 'command': "
                 "useless field in this context\n"))
 
+    def test_validate__broken_deprecated_field(self):
+        """
+        verify that ./manage.py validate shows information about deprecated fields
+        """
+        filename = os.path.join(self.tmpdir, "jobs", "broken.txt")
+        with open(filename, "wt", encoding='UTF-8') as stream:
+            print("name: broken", file=stream)
+            print("plugin: manual", file=stream)
+            print("description: broken job definition", file=stream)
+            print("command: true", file=stream)
+        with TestIO() as test_io:
+            self.tool.main(["validate"])
+        self.assertEqual(
+            test_io.stdout, (
+                "jobs/broken.txt:1-4: job '2014.com.example::broken', field 'name': "
+                "usage of deprecated field\n"))
+
     def test_info(self):
         """
         verify that ./manage.py info shows basic provider information
@@ -360,7 +384,7 @@ class ProviderManagerToolTests(TestCase):
         os.mkdir(os.path.join(tmpdir, "jobs"))
         filename = os.path.join(tmpdir, "jobs", "jobs.txt")
         with open(filename, "wt", encoding='UTF-8') as stream:
-            print("name: dummy", file=stream)
+            print("id: dummy", file=stream)
             print("plugin: shell", file=stream)
             print("command: true", file=stream)
         os.mkdir(os.path.join(tmpdir, "whitelists"))
@@ -377,6 +401,10 @@ class ProviderManagerToolTests(TestCase):
             print("#!/bin/sh", file=stream)
             print(":", file=stream)
         os.chmod(filename, 0o755)
+        os.mkdir(os.path.join(tmpdir, "src"))
+        filename = os.path.join(tmpdir, "src", "hello.c")
+        with open(filename, "wt", encoding='UTF-8') as stream:
+            print("int main() { return 0; }", file=stream)
         definition = Provider1Definition()
         definition.location = tmpdir
         definition.name = "2014.com.example:test"
@@ -433,3 +461,34 @@ class ProviderManagerToolTests(TestCase):
         with tarfile.open(tarball, "r:*") as tar:
             with self.assertRaises(KeyError):
                 tar.getmember(member)
+
+
+class ExtensionTests(TestCase):
+    """
+    Test cases for the manage_py_extension decorator
+    """
+
+    def setUp(self):
+        self.saved = ProviderManagerTool._SUB_COMMANDS[:]
+
+    def tearDown(self):
+        ProviderManagerTool._SUB_COMMANDS[:] = self.saved
+
+    def test_add_new_command(self):
+        @manage_py_extension
+        class NewCommand(ManageCommand):
+            """
+            Some new command
+            """
+        self.assertIn(NewCommand, ProviderManagerTool._SUB_COMMANDS)
+
+    def test_replace_command(self):
+        self.assertIn(InstallCommand, ProviderManagerTool._SUB_COMMANDS)
+
+        @manage_py_extension
+        class BetterInstallCommand(InstallCommand):
+            """
+            Improved version of an existing command
+            """
+        self.assertNotIn(InstallCommand, ProviderManagerTool._SUB_COMMANDS)
+        self.assertIn(BetterInstallCommand, ProviderManagerTool._SUB_COMMANDS)
